@@ -25,6 +25,7 @@ from time import sleep
 import requests
 import math
 from inorbit_edge.utils import encode_floating_point_list
+import certifi
 
 
 INORBIT_CLOUD_SDK_ROBOT_CONFIG_URL = "https://control.inorbit.ai/cloud_sdk_robot_config"
@@ -113,6 +114,7 @@ class RobotSession:
         # Register MQTT client callbacks
         self.client.on_connect = self._on_connect
         self.client.on_message = self._on_message
+        self.client.on_disconnect = self._on_disconnect
 
         # Internal variables for configuring throttling
         # The throttling is done by method instead of by topic because the same topic
@@ -285,6 +287,22 @@ class RobotSession:
                 self.logger.error("Unexpected error while processing message.")
                 raise
 
+    def _on_disconnect(self, client, userdata, rc):
+        """MQTT client disconnect callback.
+
+        Args:
+            client:     the client instance for this callback
+            userdata:   the private user data as set in Client() or userdata_set()
+            rc:         disconnection result
+        """
+
+        if rc != 0:
+            self.logger.warn(
+                "Unexpected disconnection: {}".format(mqtt.error_string(rc))
+            )
+        else:
+            self.logger.info("Disconnected from MQTT broker")
+
     def _get_custom_command_topic(self):
         return self._get_robot_subtopic(subtopic=MQTT_SUBTOPIC_CUSTOM_COMMAND)
 
@@ -395,9 +413,7 @@ class RobotSession:
         # TODO: add support for user-provided CA certificate file.
         if self.use_ssl:
             self.logger.debug("Configuring client to use SSL")
-            self.client.tls_set(
-                "/etc/ssl/certs/ca-certificates.crt", tls_version=ssl.PROTOCOL_TLSv1_2
-            )
+            self.client.tls_set(certifi.where(), tls_version=ssl.PROTOCOL_TLSv1_2)
 
         # Configure MQTT client hostname and port
         hostname = robot_config["hostname"]
@@ -408,7 +424,6 @@ class RobotSession:
         )
         self.client.connect(hostname, port, keepalive=10)
         self.client.loop_start()
-
         self._wait_for_connection_state(self._is_connected)
 
         self.logger.info(
@@ -431,8 +446,6 @@ class RobotSession:
 
         self._wait_for_connection_state(self._is_disconnected)
 
-        self.logger.info("Disconnected from MQTT broker")
-
     def publish(self, topic, message, qos=0, retain=False):
         """MQTT client wrapper method for publishing messages
 
@@ -445,7 +458,21 @@ class RobotSession:
         Returns:
             MQTTMessageInfo: Returns a MQTTMessageInfo class
         """
-        return self.client.publish(topic=topic, payload=message, qos=qos, retain=retain)
+        try:
+            info = self.client.publish(
+                topic=topic, payload=message, qos=qos, retain=retain
+            )
+        except ValueError:
+            self.logger.error("Payload greater than 268435455 bytes is not allowed")
+
+        if info.rc != mqtt.MQTT_ERR_SUCCESS:
+            self.logger.warn(
+                "There was a problem sending message {}: {}".format(
+                    info.mid, mqtt.error_string(info.rc)
+                )
+            )
+
+        return info
 
     def publish_protobuf(self, subtopic, message, qos=0, retain=False):
         """Publish protobuf messages to this robot session subtopic.
